@@ -22,7 +22,7 @@ public:
     int unpack_string_array(char**, size_t* len) override;
     int value_count(long*) override;
     void destroy(grib_context*) override;
-    void dump(grib_dumper*) override;
+    void dump(eccodes::Dumper*) override;
     void init(const long, grib_arguments*) override;
 
 private:
