@@ -20,7 +20,7 @@ public:
     grib_accessor* create_empty_accessor() override { return new grib_accessor_g1forecastmonth_t{}; }
     int pack_long(const long* val, size_t* len) override;
     int unpack_long(long* val, size_t* len) override;
-    void dump(grib_dumper*) override;
+    void dump(eccodes::Dumper*) override;
     void init(const long, grib_arguments*) override;
 
 private:
