@@ -19,6 +19,6 @@ public:
         grib_accessor_unsigned_t() { class_name_ = "section_length"; }
     grib_accessor* create_empty_accessor() override { return new grib_accessor_section_length_t{}; }
     int value_count(long*) override;
-    void dump(grib_dumper*) override;
+    void dump(eccodes::Dumper*) override;
     void init(const long, grib_arguments*) override;
 };
