@@ -23,7 +23,7 @@ public:
     long byte_count() override;
     long byte_offset() override;
     long next_offset() override;
-    void dump(grib_dumper*) override;
+    void dump(eccodes::Dumper*) override;
     void init(const long, grib_arguments*) override;
     void update_size(size_t) override;
     int compare(grib_accessor*) override;
